searchData={"items":[{"type":"module","title":"Exth","doc":"# Exth\n\nExth is an Elixir client for interacting with EVM-compatible blockchain nodes\nvia JSON-RPC. It provides a robust, type-safe interface for making Ethereum RPC\ncalls.","ref":"Exth.html"},{"type":"module","title":"Features - Exth","doc":"- 🔒 **Type Safety**: Comprehensive type specs and validation\n- 🔄 **Transport Agnostic**: Pluggable transport system (HTTP, WebSocket, IPC)\n- 🎯 **Smart Defaults**: Sensible defaults with full configurability\n- 🛡️ **Error Handling**: Detailed error reporting and recovery\n- 📦 **Batch Support**: Efficient batch request processing\n- 🔌 **Protocol Compliance**: Full JSON-RPC 2.0 specification support\n- ⚙️ **Dynamic Configuration**: Flexible configuration through both inline options and application config","ref":"Exth.html#module-features"},{"type":"module","title":"Installation - Exth","doc":"Add `exth` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:exth, \"~> 0.1.0\"},\n    # Optional dependencies:\n    # Mint for Tesla adapter\n    {:mint, \"~> 1.7\"}\n  ]\nend\n```","ref":"Exth.html#module-installation"},{"type":"module","title":"Usage - Exth","doc":"Exth offers two ways to interact with EVM nodes:\n\n1. **Provider** (High-Level): Define a provider module with convenient function\n   names and no need to pass client references.\n2. **RPC Client** (Low-Level): Direct client usage with more control, requiring\n   explicit client handling.\n\n<!-- tabs-open -->","ref":"Exth.html#module-usage"},{"type":"module","title":"Provider (Recommended) - Exth","doc":"```elixir\n# Basic usage with inline configuration\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :your_otp_app,\n    transport_type: :http,\n    rpc_url: \"https://YOUR-RPC-URL\"\nend\n\n# Dynamic configuration through application config\n# In your config/config.exs or similar:\nconfig :your_otp_app, MyProvider,\n  rpc_url: \"https://YOUR-RPC-URL\",\n  timeout: 30_000,\n  max_retries: 3\n\n# Then in your provider module:\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :your_otp_app,\n    transport_type: :http\nend\n\n# Configuration is merged with inline options taking precedence\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :your_otp_app,\n    transport_type: :http,\n    rpc_url: \"https://OVERRIDE-RPC-URL\" # This will override the config value\nend\n\n# Use the provider\n{:ok, block_number} = MyProvider.block_number()\n\n{:ok, balance} = MyProvider.get_balance(\n  \"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\",\n  \"latest\"\n)\n\n{:ok, block} = MyProvider.get_block_by_number(\"0x1\", true)\n\n{:ok, tx_hash} = MyProvider.send_raw_transaction(\"0x...\")\n```\n\nThe Provider approach is recommended for most use cases as it provides:\n\n- ✨ Clean, intuitive function names\n- 🔒 Type-safe parameters\n- 📝 Better documentation and IDE support\n- 🎯 No need to manage client references\n- ⚙️ Flexible configuration through both inline options and application config\n\n#### Configuration Options\n\nProviders can be configured through both inline options and application config.\nInline options take precedence over application config. Here are the available options:\n\n```elixir\n# Required options\ntransport_type: :http | :custom  # Transport type to use\nrpc_url: \"https://...\"          # RPC endpoint URL\n\n# Required inline option\notp_app: :your_otp_app          # Application name for config lookup\n\n# Custom transport options\nmodule: MyCustomTransport       # Required when transport_type is :custom\n\n# Optional http options\ntimeout: 30_000                 # Request timeout in milliseconds\nheaders: [{\"header\", \"value\"}]  # Custom headers for HTTP transport\n```","ref":"Exth.html#module-provider-recommended"},{"type":"module","title":"RPC Client - Exth","doc":"```elixir\nalias Exth.Rpc\n\n# 1. Define a client\n{:ok, client} = Rpc.new_client(\n  transport_type: :http,\n  rpc_url: \"https://YOUR-RPC-URL\"\n)\n\n# 2.1. Make RPC calls with explicit client\nrequest1 = Rpc.request(client, \"eth_blockNumber\", [])\n{:ok, block_number} = Rpc.send(client, request1)\n\n# 2.2. Or make RPC calls without a client\nrequest2 = Rpc.request(\n  \"eth_getBalance\",\n  [\"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\", \"latest\"]\n)\n{:ok, balance} = Rpc.send(client, request2)\n\n# 3. You can also send multiple requests in one call\nrequests = [request1, request2]\n{:ok, responses} = Rpc.send(client, requests)\n\n# 4. You can invert the order of the arguments and pipe\nRpc.request(\"eth_blockNumber\", [])\n|> Rpc.send(client)\n\n# OR\n[request1, request2]\n|> Rpc.send(client)\n```\n\nUse the RPC Client approach when you need:\n\n- 🔧 Direct control over RPC calls\n- 🔄 Dynamic method names\n- 🛠️ Custom parameter handling\n- 🎛️ Flexible client management (multiple clients, runtime configuration)\n\n<!-- tabs-close -->","ref":"Exth.html#module-rpc-client"},{"type":"module","title":"Transport Options - Exth","doc":"Exth uses a pluggable transport system that supports different communication\nprotocols. Each transport type can be configured with specific options:\n\n<!-- tabs-open -->","ref":"Exth.html#module-transport-options"},{"type":"module","title":"HTTP Transport - Exth","doc":"The default HTTP transport is built on Tesla, providing a robust HTTP client\nwith middleware support:\n\n```elixir\n# Provider configuration\ndefmodule MyProvider do\n  use Exth.Provider,\n    transport_type: :http,\n    rpc_url: \"https://eth-mainnet.example.com\",\n    # Optional HTTP-specific configuration\n    adapter: Tesla.Adapter.Mint, # Default HTTP adapter\n    headers: [{\"authorization\", \"Bearer token\"}],\n    timeout: 30_000, # Request timeout in ms\nend\n\n# Direct client configuration\n{:ok, client} = Exth.Rpc.new(\n  transport_type: :http,\n  rpc_url: \"https://eth-mainnet.example.com\",\n  adapter: Tesla.Adapter.Mint,\n  headers: [{\"authorization\", \"Bearer token\"}],\n  timeout: 30_000\n)\n```\n\n- ✨ **HTTP** (`:http`)\n\n  - Built on Tesla HTTP client\n  - Configurable adapters (Mint, Hackney, etc.)\n  - Configurable headers and timeouts","ref":"Exth.html#module-http-transport"},{"type":"module","title":"Custom Transport - Exth","doc":"Implement your own transport by creating a module and implementing the\n`Exth.Transport.Transportable` protocol:\n\n```elixir\ndefmodule MyCustomTransport do\n  # Transport struct should be whatever you need\n  defstruct [:config]\nend\n\ndefimpl Exth.Transport.Transportable, for: MyCustomTransport do\n  def new(transport, opts) do\n    # Initialize your transport configuration\n    %MyCustomTransport{config: opts}\n  end\n\n  def call(transport, request) do\n    # Handle the JSON-RPC request\n    # Return {:ok, response} or {:error, reason}\n  end\nend\n\n# Use your custom transport\ndefmodule MyProvider do\n  use Exth.Provider,\n    transport_type: :custom,\n    module: MyCustomTransport,\n    rpc_url: \"custom://endpoint\",\n    # Additional custom options\n    custom_option: \"value\"\nend\n\n# Direct client configuration\n{:ok, client} = Exth.Rpc.new_request(\n  transport_type: :custom,\n  rpc_url: \"https://eth-mainnet.example.com\",\n  module: MyCustomTransport,\n  custom_option: \"value\"\n)\n```\n\n- 🔧 **Custom** (`:custom`)\n  - Full control over transport implementation\n  - Custom state management\n\n<!-- tabs-close -->","ref":"Exth.html#module-custom-transport"},{"type":"module","title":"Examples - Exth","doc":"Check out our [examples](https://github.com/joaop21/exth/tree/main/examples)\ndirectory for practical usage examples","ref":"Exth.html#module-examples"},{"type":"module","title":"Requirements - Exth","doc":"- Elixir ~> 1.18\n- Erlang/OTP 26 or later","ref":"Exth.html#module-requirements"},{"type":"module","title":"Contributing - Exth","doc":"1. Fork it\n2. Create your feature branch (`git checkout -b feature/my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin feature/my-new-feature`)\n5. Create new Pull Request","ref":"Exth.html#module-contributing"},{"type":"module","title":"License - Exth","doc":"This project is licensed under the MIT License. See [LICENSE](LICENSE) for details.","ref":"Exth.html#module-license"},{"type":"module","title":"Exth.Provider","doc":"Provides a macro for generating Ethereum JSON-RPC client methods with built-in client caching.\n\nThis module automatically generates functions for common Ethereum JSON-RPC methods,\nhandling the client lifecycle, request formatting, and response parsing.","ref":"Exth.Provider.html"},{"type":"module","title":"Features - Exth.Provider","doc":"* Automatic client caching and reuse\n  * Standard Ethereum JSON-RPC method implementations\n  * Type-safe function signatures\n  * Consistent error handling\n  * Automatic request ID management\n  * Connection pooling and reuse\n  * Configurable transport layer\n  * Dynamic configuration through both inline options and application config","ref":"Exth.Provider.html#module-features"},{"type":"module","title":"Architecture - Exth.Provider","doc":"The Provider module works as follows:\n\n1. When `use Exth.Provider` is called, it:\n   * Validates the required configuration options\n   * Merges inline options with application config (inline takes precedence)\n   * Generates a set of standardized RPC method functions\n   * Sets up client caching mechanisms\n\n2. For each RPC call:\n   * Reuses cached clients when possible\n   * Automatically formats parameters\n   * Handles request/response lifecycle\n   * Provides consistent error handling","ref":"Exth.Provider.html#module-architecture"},{"type":"module","title":"Usage - Exth.Provider","doc":"","ref":"Exth.Provider.html#module-usage"},{"type":"module","title":"Basic Usage - Exth.Provider","doc":"```elixir\ndefmodule MyProvider do\n  use Exth.Provider,\n    transport_type: :http,\n    rpc_url: \"https://my-eth-node.com\"\nend\n\n# Then use the generated functions\n{:ok, balance} = MyProvider.eth_getBalance(\"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\")\n{:ok, block} = MyProvider.eth_getBlockByNumber(12345)\n```","ref":"Exth.Provider.html#module-basic-usage"},{"type":"module","title":"Dynamic Configuration - Exth.Provider","doc":"You can configure providers through both inline options and application config.\nInline options take precedence over application config.\n\n```elixir\n# In your config/config.exs or similar:\nconfig :my_app, MyProvider,\n  rpc_url: \"https://config-rpc-url\",\n  timeout: 30_000\n\n# In your provider module:\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :my_app,\n    transport_type: :http,\n    rpc_url: \"https://override-rpc-url\" # This will override the config value\nend\n```","ref":"Exth.Provider.html#module-dynamic-configuration"},{"type":"module","title":"Configuration Options - Exth.Provider","doc":"","ref":"Exth.Provider.html#module-configuration-options"},{"type":"module","title":"Required Options - Exth.Provider","doc":"* `:transport_type` - The transport type to use (`:http` or `:custom`)\n  * `:rpc_url` - The URL of the Ethereum JSON-RPC endpoint\n  * `:otp_app` - The application name for config lookup (required when using application config)","ref":"Exth.Provider.html#module-required-options"},{"type":"module","title":"Generated Functions - Exth.Provider","doc":"All generated functions follow these conventions:\n\n  * Return `{:ok, result}` for successful calls\n  * Return `{:error, reason}` for failures\n  * Accept an optional `block_tag` parameter (defaults to \"latest\") where applicable","ref":"Exth.Provider.html#module-generated-functions"},{"type":"module","title":"Error Handling - Exth.Provider","doc":"Possible error responses:\n  * `{:error, %{code: code, message: msg}}` - RPC method error\n  * `{:error, reason}` - Other errors with description","ref":"Exth.Provider.html#module-error-handling"},{"type":"module","title":"Examples - Exth.Provider","doc":"```elixir\n# Get balance for specific block\n{:ok, balance} = MyProvider.eth_getBalance(\n  \"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\",\n  \"0x1b4\"\n)\n\n# Get latest block\n{:ok, block} = MyProvider.eth_getBlockByNumber(\"latest\", true)\n\n# Send raw transaction\n{:ok, tx_hash} = MyProvider.eth_sendRawTransaction(\"0x...\")\n```\n\nSee `Exth.Provider.Methods` for a complete list of available RPC methods.","ref":"Exth.Provider.html#module-examples"},{"type":"macro","title":"Exth.Provider.__using__/1","doc":"Implements the provider behavior in the using module.\n\nThis macro is the entry point for creating an Ethereum JSON-RPC provider.\nIt validates the provided options and generates all the necessary functions\nfor interacting with an Ethereum node.","ref":"Exth.Provider.html#__using__/1"},{"type":"macro","title":"Options - Exth.Provider.__using__/1","doc":"See the moduledoc for complete configuration options.","ref":"Exth.Provider.html#__using__/1-options"},{"type":"macro","title":"Examples - Exth.Provider.__using__/1","doc":"```elixir\ndefmodule MyProvider do\n  use Exth.Provider,\n    transport_type: :http,\n    rpc_url: \"https://mainnet.infura.io/v3/YOUR-PROJECT-ID\"\nend\n```","ref":"Exth.Provider.html#__using__/1-examples"},{"type":"macro","title":"Exth.Provider.generate_get_client/0","doc":"","ref":"Exth.Provider.html#generate_get_client/0"},{"type":"macro","title":"Exth.Provider.generate_handle_response/0","doc":"","ref":"Exth.Provider.html#generate_handle_response/0"},{"type":"macro","title":"Exth.Provider.generate_provider/1","doc":"Generates the provider module with all RPC method implementations.\n\nThis macro creates the actual provider implementation by:\n  * Setting up the client cache\n  * Generating functions for each RPC method\n  * Implementing response handling\n  * Creating helper functions","ref":"Exth.Provider.html#generate_provider/1"},{"type":"macro","title":"Parameters - Exth.Provider.generate_provider/1","doc":"* `opts` - Keyword list of provider options","ref":"Exth.Provider.html#generate_provider/1-parameters"},{"type":"macro","title":"Generated Functions - Exth.Provider.generate_provider/1","doc":"For each RPC method defined in `Exth.Provider.Methods`, this macro generates:\n  * A public function with proper type specs\n  * Documentation with parameters and return values\n  * Automatic parameter validation\n  * Response handling and formatting","ref":"Exth.Provider.html#generate_provider/1-generated-functions"},{"type":"macro","title":"Exth.Provider.generate_rpc_method/4","doc":"","ref":"Exth.Provider.html#generate_rpc_method/4"},{"type":"module","title":"Exth.Rpc","doc":"Core module for making JSON-RPC requests to EVM-compatible blockchain nodes.\n\nThis module provides a comprehensive interface for interacting with JSON-RPC endpoints,\noffering flexible request building, multiple transport options, and robust error handling.","ref":"Exth.Rpc.html"},{"type":"module","title":"Key Features - Exth.Rpc","doc":"* Client-based and standalone request building\n  * Batch request support for better performance\n  * Configurable transport layers (HTTP, custom implementations)\n  * Automatic request ID generation and version management\n\n* Comprehensive error handling","ref":"Exth.Rpc.html#module-key-features"},{"type":"module","title":"Quick Start - Exth.Rpc","doc":"# Create a client\n    client = Rpc.new_client(:http, rpc_url: \"https://eth-mainnet.example.com\")\n\n    # Make a simple request\n    request = Rpc.request(\"eth_blockNumber\", [])\n    {:ok, response} = Rpc.send(client, request)","ref":"Exth.Rpc.html#module-quick-start"},{"type":"module","title":"Request Patterns - Exth.Rpc","doc":"","ref":"Exth.Rpc.html#module-request-patterns"},{"type":"module","title":"Client-based Requests - Exth.Rpc","doc":"# Create client once and reuse\n    client = Rpc.new_client(:http, rpc_url: \"https://eth-mainnet.example.com\")\n    \n    # Make requests with client\n    request = Rpc.request(client, \"eth_getBalance\", [\"0x742d...\", \"latest\"])\n    {:ok, response} = Rpc.send(client, request)","ref":"Exth.Rpc.html#module-client-based-requests"},{"type":"module","title":"Standalone Requests - Exth.Rpc","doc":"# Build requests without client\n    request = Rpc.request(\"eth_blockNumber\", [])\n    {:ok, response} = Rpc.send(client, request)","ref":"Exth.Rpc.html#module-standalone-requests"},{"type":"module","title":"Batch Requests - Exth.Rpc","doc":"# Send multiple requests in one call\n    requests = [\n      Rpc.request(\"eth_blockNumber\", []),\n      Rpc.request(\"eth_gasPrice\", [])\n    ]\n    {:ok, [block_response, gas_response]} = Rpc.send(client, requests)","ref":"Exth.Rpc.html#module-batch-requests"},{"type":"module","title":"Transport Configuration - Exth.Rpc","doc":"The module supports different transport layers:\n\n  * `:http` - Standard HTTP/HTTPS transport with configurable headers and timeouts\n  * `:custom` - Custom transport implementations for special needs","ref":"Exth.Rpc.html#module-transport-configuration"},{"type":"module","title":"Error Handling - Exth.Rpc","doc":"case Rpc.send(client, request) do\n      {:ok, response} -> \n        # Handle successful response\n        handle_success(response.result)\n      \n      {:error, %{code: code, message: msg}} -> \n        # Handle RPC-level errors\n        handle_rpc_error(code, msg)\n      \n      {:error, %Exception{} = e} -> \n        # Handle transport-level errors\n        handle_transport_error(e)\n    end\n\nSee the individual function documentation for more detailed usage examples\nand options.","ref":"Exth.Rpc.html#module-error-handling"},{"type":"type","title":"Exth.Rpc.id/0","doc":"","ref":"Exth.Rpc.html#t:id/0"},{"type":"type","title":"Exth.Rpc.jsonrpc/0","doc":"","ref":"Exth.Rpc.html#t:jsonrpc/0"},{"type":"function","title":"Exth.Rpc.jsonrpc_version/0","doc":"Returns the JSON-RPC protocol version used by the client.","ref":"Exth.Rpc.html#jsonrpc_version/0"},{"type":"type","title":"Exth.Rpc.method/0","doc":"","ref":"Exth.Rpc.html#t:method/0"},{"type":"function","title":"Exth.Rpc.new_client/2","doc":"Creates a new JSON-RPC client with the specified transport type and options.","ref":"Exth.Rpc.html#new_client/2"},{"type":"function","title":"Transport Types - Exth.Rpc.new_client/2","doc":"* `:http` - HTTP/HTTPS transport\n  * `:custom` - Custom transport implementation","ref":"Exth.Rpc.html#new_client/2-transport-types"},{"type":"function","title":"Options - Exth.Rpc.new_client/2","doc":"* `:rpc_url` - (Required) The endpoint URL\n  * `:headers` - Additional HTTP headers (HTTP transport only)\n  * `:timeout` - Request timeout in ms (HTTP transport only)","ref":"Exth.Rpc.html#new_client/2-options"},{"type":"function","title":"Examples - Exth.Rpc.new_client/2","doc":"client = Rpc.new_client(:http, rpc_url: \"https://eth-mainnet.example.com\")","ref":"Exth.Rpc.html#new_client/2-examples"},{"type":"type","title":"Exth.Rpc.params/0","doc":"","ref":"Exth.Rpc.html#t:params/0"},{"type":"function","title":"Exth.Rpc.request/2","doc":"Builds a new JSON-RPC request without requiring a client instance.\n\nThis is a convenience function for creating requests that can be later used\nwith a client. Useful when building multiple requests before sending them.\n\nWhen creating a standalone request, it will:\n  * Set the protocol version to \"2.0\"\n  * Leave the request ID as nil (it will be set when used with a client)\n  * Format parameters according to the JSON-RPC spec\n\nThe ID-less request allows for flexibility, as the actual ID will be assigned\nby the client when the request is sent, ensuring proper ID sequencing within\nthe client's context.","ref":"Exth.Rpc.html#request/2"},{"type":"function","title":"Examples - Exth.Rpc.request/2","doc":"# Create standalone requests - no IDs yet\n    request1 = Rpc.request(\"eth_blockNumber\", [])\n    # => %Request{id: nil, method: \"eth_blockNumber\", params: []}\n\n    request2 = Rpc.request(\"eth_getBalance\", [\"0x742d...\", \"latest\"])\n    # => %Request{id: nil, method: \"eth_getBalance\", params: [\"0x742d...\", \"latest\"]}\n\n    # When sent with a client, IDs will be assigned\n    {:ok, response} = Rpc.send(client, request1)\n    # request1 now has id: 1\n\n    # In batch requests, each request gets a sequential ID\n    {:ok, responses} = Rpc.send(client, [request1, request2])\n    # request1 has id: 2, request2 has id: 3","ref":"Exth.Rpc.html#request/2-examples"},{"type":"function","title":"Exth.Rpc.request/3","doc":"Builds a new JSON-RPC request using a client for the given method and parameters.\n\nWhen using a client, the request will automatically:\n  * Set the protocol version to \"2.0\"\n  * Generate a unique request ID based on the client's ID sequence\n  * Format parameters according to the JSON-RPC spec\n\nThe client-based request ensures that each request gets a unique ID within\nthe client's context, which is important for correlating responses in batch\nrequests or concurrent operations.","ref":"Exth.Rpc.html#request/3"},{"type":"function","title":"Examples - Exth.Rpc.request/3","doc":"# Simple request with no parameters - includes client-generated ID\n    request = Rpc.request(client, \"eth_blockNumber\", [])\n    # => %Request{id: 1, method: \"eth_blockNumber\", params: []}\n\n    # Request with multiple parameters - next ID in sequence\n    request = Rpc.request(client, \"eth_getBalance\", [\"0x742d...\", \"latest\"])\n    # => %Request{id: 2, method: \"eth_getBalance\", params: [\"0x742d...\", \"latest\"]}\n\n    # The request can be later used with Rpc.send/2\n    {:ok, response} = Rpc.send(client, request)","ref":"Exth.Rpc.html#request/3-examples"},{"type":"function","title":"Exth.Rpc.send/2","doc":"Sends one or more JSON-RPC requests using the client's configured transport.\n\nThis function is the main interface for executing requests. It handles both\nsingle and batch requests automatically, with proper error handling and response\ndecoding.","ref":"Exth.Rpc.html#send/2"},{"type":"function","title":"Request Types - Exth.Rpc.send/2","doc":"* Single request: Pass a single Request.t()\n  * Batch request: Pass a list of Request.t()","ref":"Exth.Rpc.html#send/2-request-types"},{"type":"function","title":"Returns - Exth.Rpc.send/2","doc":"* `{:ok, response}` - Successful single request with decoded response\n  * `{:ok, responses}` - Successful batch request with list of decoded responses\n  * `{:error, reason}` - Request failed with error details (Exception.t() or map())","ref":"Exth.Rpc.html#send/2-returns"},{"type":"function","title":"Examples - Exth.Rpc.send/2","doc":"# Single request\n    {:ok, response} = Rpc.send(client, request)\n    block_number = response.result\n\n    # Batch request for better performance\n    requests = [\n      Rpc.request(\"eth_blockNumber\", []),\n      Rpc.request(\"eth_gasPrice\", [])\n    ]\n    {:ok, [block_response, gas_response]} = Rpc.send(client, requests)\n\n    # You can also invert the arguments\n    Rpc.request(\"eth_blockNumber\", [])\n    |> Rpc.send(client)\n\n    # Error handling\n    case Rpc.send(client, request) do\n      {:ok, response} -> handle_success(response)\n      {:error, %{code: code, message: msg}} -> handle_rpc_error(code, msg)\n      {:error, %Exception{} = e} -> handle_transport_error(e)\n    end","ref":"Exth.Rpc.html#send/2-examples"},{"type":"module","title":"Exth.Transport","doc":"Factory module for creating JSON-RPC transport implementations.\n\nThis module provides a unified interface for creating and managing different transport\nmechanisms (HTTP, WebSocket, IPC) for JSON-RPC communication with EVM nodes.","ref":"Exth.Transport.html"},{"type":"module","title":"Features - Exth.Transport","doc":"* Pluggable transport system via the `Transportable` protocol\n  * Built-in HTTP transport with Tesla/Mint\n  * Consistent interface across transport types\n  * Configurable timeout and retry mechanisms\n  * Transport-specific option handling","ref":"Exth.Transport.html#module-features"},{"type":"module","title":"Transport Types - Exth.Transport","doc":"Currently supported:\n  * `:http` - HTTP/HTTPS transport using Tesla with Mint adapter\n  * `:custom` - Custom transport implementations\n\nComing soon:\n  * `:ws` - WebSocket transport\n  * `:ipc` - Unix domain socket transport","ref":"Exth.Transport.html#module-transport-types"},{"type":"module","title":"Usage - Exth.Transport","doc":"# Create an HTTP transport\n    transport = Transport.new(:http,\n      rpc_url: \"https://mainnet.infura.io/v3/YOUR-PROJECT-ID\",\n      timeout: 30_000,\n      headers: [{\"authorization\", \"Bearer token\"}]\n    )\n\n    # Make requests\n    {:ok, response} = Transport.call(transport, request)\n    {:ok, responses} = Transport.call(transport, [request1, request2])","ref":"Exth.Transport.html#module-usage"},{"type":"module","title":"Configuration - Exth.Transport","doc":"Common options for all transports:\n\n  * `:rpc_url` - (Required) The endpoint URL\n\nHTTP-specific options:\n\n  * `:headers` - Additional HTTP headers\n  * `:timeout` - Request timeout in milliseconds (default: 30000)\n  * `:adapter` - Tesla adapter to use (default: Tesla.Adapter.Mint)","ref":"Exth.Transport.html#module-configuration"},{"type":"module","title":"Custom Transport Implementation - Exth.Transport","doc":"To implement a custom transport:\n\n1. Define your transport struct:\n\n       defmodule MyTransport do\n         defstruct [:config]\n       end\n\n2. Implement the `Transportable` protocol:\n\n       defimpl Exth.Transport.Transportable, for: MyTransport do\n         def new(_transport, opts) do\n           %MyTransport{config: opts}\n         end\n\n         def call(transport, request) do\n           # Implement request handling\n           {:ok, response}\n         end\n       end\n\n3. Use your transport:\n\n       transport = Transport.new(:custom,\n         module: MyTransport,\n         rpc_url: \"custom://endpoint\",\n         # other options...\n       )","ref":"Exth.Transport.html#module-custom-transport-implementation"},{"type":"module","title":"Error Handling - Exth.Transport","doc":"The module uses consistent error handling:\n\n  * `{:ok, response}` - Successful request with decoded response\n  * `{:ok, responses}` - Successful batch request with responses\n  * `{:error, reason}` - Request failed with detailed reason\n\nHTTP-specific errors:\n  * `{:error, {:http_error, status}}` - HTTP error response\n  * `{:error, :timeout}` - Request timeout\n  * `{:error, :network_error}` - Network connectivity issues","ref":"Exth.Transport.html#module-error-handling"},{"type":"module","title":"Best Practices - Exth.Transport","doc":"* Use appropriate timeouts for your use case\n  * Implement retry logic for transient failures\n  * Handle batch requests efficiently\n  * Monitor transport health and metrics\n  * Properly handle connection pooling\n  * Use secure transport options in production\n\nSee `Exth.Transport.Transportable` for protocol details and\n`Exth.Transport.Http` for HTTP transport specifics.","ref":"Exth.Transport.html#module-best-practices"},{"type":"function","title":"Exth.Transport.call/2","doc":"Makes a request using the configured transport.","ref":"Exth.Transport.html#call/2"},{"type":"function","title":"Parameters - Exth.Transport.call/2","doc":"* `transportable` - The configured transport instance\n  * `request` - The JSON-RPC request to send","ref":"Exth.Transport.html#call/2-parameters"},{"type":"function","title":"Returns - Exth.Transport.call/2","doc":"* `{:ok, response}` - Successful request with decoded response\n  * `{:error, reason}` - Request failed with error reason","ref":"Exth.Transport.html#call/2-returns"},{"type":"type","title":"Exth.Transport.call_response/0","doc":"","ref":"Exth.Transport.html#t:call_response/0"},{"type":"type","title":"Exth.Transport.error_reason/0","doc":"","ref":"Exth.Transport.html#t:error_reason/0"},{"type":"function","title":"Exth.Transport.new/2","doc":"Creates a new transport struct with the given type and options.","ref":"Exth.Transport.html#new/2"},{"type":"function","title":"Parameters - Exth.Transport.new/2","doc":"* `type` - The type of transport to create (`:http` or `:custom`)\n  * `opts` - Configuration options for the transport","ref":"Exth.Transport.html#new/2-parameters"},{"type":"function","title":"Returns - Exth.Transport.new/2","doc":"* A configured transport struct implementing the `Transportable` protocol","ref":"Exth.Transport.html#new/2-returns"},{"type":"function","title":"Raises - Exth.Transport.new/2","doc":"* `ArgumentError` if required options are missing or type is invalid","ref":"Exth.Transport.html#new/2-raises"},{"type":"type","title":"Exth.Transport.options/0","doc":"Transport configuration options.","ref":"Exth.Transport.html#t:options/0"},{"type":"type","title":"Exth.Transport.type/0","doc":"Supported transport types:\n* `:http` - HTTP/HTTPS transport\n* `:custom` - Custom transport implementation","ref":"Exth.Transport.html#t:type/0"},{"type":"module","title":"Exth.Provider.ClientCache","doc":"Provides caching functionality for Ethereum JSON-RPC clients.\n\nThis module implements a high-performance, persistent caching mechanism for RPC clients\nusing Erlang's `:persistent_term` storage. It ensures efficient client reuse while\nmaintaining a small memory footprint.","ref":"Exth.Provider.ClientCache.html"},{"type":"module","title":"Features - Exth.Provider.ClientCache","doc":"* Persistent client caching\n  * Thread-safe operations\n  * O(1) lookup performance\n  * Memory-efficient storage\n  * Process-independent cache\n  * Atomic operations\n  * Automatic client lifecycle","ref":"Exth.Provider.ClientCache.html#module-features"},{"type":"module","title":"Cache Architecture - Exth.Provider.ClientCache","doc":"The cache is built on `:persistent_term` and provides:\n\n  * Fast, constant-time lookups\n  * Single copy per cached term\n  * Process crash resilience\n  * Node-local storage\n  * Atomic operations\n  * Low memory overhead","ref":"Exth.Provider.ClientCache.html#module-cache-architecture"},{"type":"module","title":"Cache Structure - Exth.Provider.ClientCache","doc":"Keys:\n  * Format: `{__MODULE__, transport_type, rpc_url}`\n  * Unique per client configuration\n  * Efficient lookup pattern\n\nValues:\n  * Type: `Exth.Rpc.Client` structs\n  * Complete client configuration\n  * Ready-to-use instances","ref":"Exth.Provider.ClientCache.html#module-cache-structure"},{"type":"module","title":"Usage Examples - Exth.Provider.ClientCache","doc":"# Fetch existing client\n    {:ok, client} = ClientCache.get_client(:http, \"https://eth-mainnet.example.com\")\n\n    # Create and cache new client\n    client = ClientCache.create_client(\n      :http,\n      \"https://eth-mainnet.example.com\",\n      %Client{\n        transport: transport,\n        counter: counter\n      }\n    )\n\n    # Pattern for get-or-create\n    case ClientCache.get_client(transport_type, rpc_url) do\n      {:ok, client} ->\n        client\n      {:error, :not_found} ->\n        client = create_new_client()\n        ClientCache.create_client(transport_type, rpc_url, client)\n    end","ref":"Exth.Provider.ClientCache.html#module-usage-examples"},{"type":"module","title":"Performance Characteristics - Exth.Provider.ClientCache","doc":"Read Operations:\n  * O(1) lookup time\n  * No process communication\n  * Zero-copy term sharing\n  * Minimal memory overhead\n\nWrite Operations:\n  * Atomic updates\n  * Copy-on-write semantics\n  * Process-independent durability\n\nCache Properties:\n  * Node-local scope\n  * Process-independent\n  * Crash-resilient\n  * Memory-efficient","ref":"Exth.Provider.ClientCache.html#module-performance-characteristics"},{"type":"module","title":"Best Practices - Exth.Provider.ClientCache","doc":"* Reuse clients when possible\n  * Handle cache misses gracefully\n  * Monitor cache size\n  * Clean up unused clients\n  * Use appropriate timeouts\n  * Implement circuit breakers","ref":"Exth.Provider.ClientCache.html#module-best-practices"},{"type":"module","title":"Limitations - Exth.Provider.ClientCache","doc":"* Node-local only (not distributed)\n  * No automatic cleanup\n  * Memory limited by node\n  * No expiration mechanism","ref":"Exth.Provider.ClientCache.html#module-limitations"},{"type":"module","title":"Error Handling - Exth.Provider.ClientCache","doc":"* `{:ok, client}` - Client found in cache\n  * `{:error, :not_found}` - Cache miss\n  * Raises `ArgumentError` for invalid inputs","ref":"Exth.Provider.ClientCache.html#module-error-handling"},{"type":"module","title":"Implementation Details - Exth.Provider.ClientCache","doc":"The cache uses Erlang's `:persistent_term` module which:\n  * Stores terms in a special ETS table\n  * Provides fast, copy-free reads\n  * Maintains single source of truth\n  * Survives process crashes\n  * Uses minimal memory\n\nSee `Exth.Rpc.Client` for client details and\n`Exth.Transport` for transport configuration.","ref":"Exth.Provider.ClientCache.html#module-implementation-details"},{"type":"function","title":"Exth.Provider.ClientCache.create_client/3","doc":"Creates and caches a new RPC client for the given transport type and URL.\n\nStores the client in persistent term storage and returns the cached client.\nIf a client already exists for the given combination, it will be overwritten.","ref":"Exth.Provider.ClientCache.html#create_client/3"},{"type":"function","title":"Parameters - Exth.Provider.ClientCache.create_client/3","doc":"* `transport_type` - The type of transport (e.g., `:http`, `:websocket`)\n  * `rpc_url` - The URL of the Ethereum JSON-RPC endpoint\n  * `client` - The `Client` struct to cache","ref":"Exth.Provider.ClientCache.html#create_client/3-parameters"},{"type":"function","title":"Returns - Exth.Provider.ClientCache.create_client/3","doc":"* The cached client instance","ref":"Exth.Provider.ClientCache.html#create_client/3-returns"},{"type":"function","title":"Examples - Exth.Provider.ClientCache.create_client/3","doc":"iex> client = %Client{}\n    iex> ClientCache.create_client(:http, \"https://eth-mainnet.example.com\", client)\n    %Client{}","ref":"Exth.Provider.ClientCache.html#create_client/3-examples"},{"type":"function","title":"Exth.Provider.ClientCache.get_client/2","doc":"Retrieves a cached RPC client for the given transport type and URL.\n\nAttempts to fetch an existing client from the persistent term storage.\nReturns `{:error, :not_found}` if no client exists for the given combination.","ref":"Exth.Provider.ClientCache.html#get_client/2"},{"type":"function","title":"Parameters - Exth.Provider.ClientCache.get_client/2","doc":"* `transport_type` - The type of transport (e.g., `:http`, `:websocket`)\n  * `rpc_url` - The URL of the Ethereum JSON-RPC endpoint","ref":"Exth.Provider.ClientCache.html#get_client/2-parameters"},{"type":"function","title":"Returns - Exth.Provider.ClientCache.get_client/2","doc":"* `{:ok, client}` - When a cached client is found\n  * `{:error, :not_found}` - When no cached client exists","ref":"Exth.Provider.ClientCache.html#get_client/2-returns"},{"type":"function","title":"Examples - Exth.Provider.ClientCache.get_client/2","doc":"iex> ClientCache.get_client(:http, \"https://eth-mainnet.example.com\")\n    {:ok, %Client{}}\n\n    iex> ClientCache.get_client(:http, \"https://non-existing.example.com\")\n    {:error, :not_found}","ref":"Exth.Provider.ClientCache.html#get_client/2-examples"},{"type":"module","title":"Exth.Provider.Methods","doc":"Defines the available Ethereum JSON-RPC methods and their parameters.\n\nThis module maintains a comprehensive mapping of supported Ethereum JSON-RPC methods,\ntheir parameter requirements, and whether they accept a default block parameter.","ref":"Exth.Provider.Methods.html"},{"type":"module","title":"Method Structure - Exth.Provider.Methods","doc":"Each method is defined as a tuple with three elements:\n  * RPC method name (atom)\n  * List of required parameters (list of atoms)\n  * Boolean indicating if method accepts a block parameter (boolean)","ref":"Exth.Provider.Methods.html#module-method-structure"},{"type":"module","title":"Method Categories - Exth.Provider.Methods","doc":"Methods are organized into the following namespaces:","ref":"Exth.Provider.Methods.html#module-method-categories"},{"type":"module","title":"Web3 Namespace - Exth.Provider.Methods","doc":"* Basic web3 operations (clientVersion, sha3)","ref":"Exth.Provider.Methods.html#module-web3-namespace"},{"type":"module","title":"Net Namespace - Exth.Provider.Methods","doc":"* Network status and information\n  * Peer connections\n  * Network version","ref":"Exth.Provider.Methods.html#module-net-namespace"},{"type":"module","title":"Eth Namespace - Exth.Provider.Methods","doc":"* Basic chain information\n  * Block operations\n  * Transaction operations\n  * Account operations\n  * Filter management\n  * Contract interaction","ref":"Exth.Provider.Methods.html#module-eth-namespace"},{"type":"module","title":"Parameter Types - Exth.Provider.Methods","doc":"Common parameter types include:\n  * `:address` - 20-byte Ethereum address (hex string)\n  * `:block_hash` - 32-byte block hash (hex string)\n  * `:block_number` - Block number or tag (\"latest\", \"earliest\", \"pending\")\n  * `:transaction_hash` - 32-byte transaction hash (hex string)\n  * `:data` - Arbitrary hex data\n  * `:filter` - Filter options object\n  * `:transaction` - Transaction object\n  * `:position` - Storage position (hex string)","ref":"Exth.Provider.Methods.html#module-parameter-types"},{"type":"module","title":"Usage - Exth.Provider.Methods","doc":"This module is primarily used internally by the Provider module to:\n  * Generate method implementations\n  * Validate method parameters\n  * Handle default block parameters\n\nTo get the complete list of available methods:\n\n    iex> Exth.Provider.Methods.methods()\n    %{\n      client_version: {:web3_clientVersion, [], false},\n      # ... other methods\n    }","ref":"Exth.Provider.Methods.html#module-usage"},{"type":"function","title":"Exth.Provider.Methods.methods/0","doc":"Returns a map of all supported Ethereum JSON-RPC methods.\n\nEach method entry contains:\n  * Key: The friendly method name used in the generated functions\n  * Value: A tuple of {rpc_method_name, parameter_list, accepts_block_param}","ref":"Exth.Provider.Methods.html#methods/0"},{"type":"function","title":"Returns - Exth.Provider.Methods.methods/0","doc":"* Map of method definitions","ref":"Exth.Provider.Methods.html#methods/0-returns"},{"type":"function","title":"Examples - Exth.Provider.Methods.methods/0","doc":"iex> Exth.Provider.Methods.methods()\n    %{\n      client_version: {:web3_clientVersion, [], false},\n      get_balance: {:eth_getBalance, [:address], true},\n      # ... other methods\n    }","ref":"Exth.Provider.Methods.html#methods/0-examples"},{"type":"module","title":"Exth.Rpc.Client","doc":"Core client module for making JSON-RPC requests to EVM nodes.\n\nThis module provides the main client interface for interacting with EVM nodes,\nhandling request creation, response parsing, and client lifecycle management.","ref":"Exth.Rpc.Client.html"},{"type":"module","title":"Features - Exth.Rpc.Client","doc":"* Atomic request ID generation\n  * Transport abstraction\n  * Request/response lifecycle management\n  * Batch request support\n  * Error handling","ref":"Exth.Rpc.Client.html#module-features"},{"type":"module","title":"Usage - Exth.Rpc.Client","doc":"# Create a new client\n    client = Client.new(:http,\n      rpc_url: \"https://eth-mainnet.example.com\",\n      timeout: 30_000\n    )\n\n    # Create a request\n    request = Client.request(client, \"eth_blockNumber\", [])\n\n    # Send the request\n    {:ok, response} = Client.send(client, request)\n\n    # Send batch requests\n    {:ok, responses} = Client.send(client, [request1, request2])","ref":"Exth.Rpc.Client.html#module-usage"},{"type":"module","title":"Client Configuration - Exth.Rpc.Client","doc":"The client accepts the following options:\n\n  * `:rpc_url` - (Required) The endpoint URL\n  * `:transport_type` - Transport to use (`:http` or `:custom`)\n  * `:timeout` - Request timeout in milliseconds\n  * `:headers` - Additional HTTP headers (HTTP only)","ref":"Exth.Rpc.Client.html#module-client-configuration"},{"type":"module","title":"Request ID Generation - Exth.Rpc.Client","doc":"The client uses Erlang's `:atomics` for thread-safe, monotonic request ID\ngeneration. This ensures:\n\n  * Unique IDs across concurrent requests\n  * No ID collisions in batch requests\n  * Efficient ID allocation\n  * Process-independent ID tracking","ref":"Exth.Rpc.Client.html#module-request-id-generation"},{"type":"module","title":"Transport Layer - Exth.Rpc.Client","doc":"The client supports different transport mechanisms through the\n`Exth.Transport.Transportable` protocol:\n\n  * Built-in HTTP transport using Tesla/Mint\n  * Custom transport implementations\n  * Future support for WebSocket and IPC","ref":"Exth.Rpc.Client.html#module-transport-layer"},{"type":"module","title":"Error Handling - Exth.Rpc.Client","doc":"The client provides consistent error handling:\n\n  * `{:ok, response}` - Successful request\n  * `{:error, reason}` - Request failed","ref":"Exth.Rpc.Client.html#module-error-handling"},{"type":"module","title":"Best Practices - Exth.Rpc.Client","doc":"* Reuse client instances when possible\n  * Use batch requests for multiple calls\n  * Implement appropriate timeouts\n  * Handle errors gracefully\n  * Monitor client health\n  * Clean up resources when done","ref":"Exth.Rpc.Client.html#module-best-practices"},{"type":"module","title":"Examples - Exth.Rpc.Client","doc":"# Basic request\n    client = Client.new(:http, rpc_url: \"https://eth-mainnet.example.com\")\n    request = Client.request(client, \"eth_blockNumber\", [])\n    {:ok, block_number} = Client.send(client, request)\n\n    # Batch request\n    requests = [\n      Client.request(client, \"eth_blockNumber\", []),\n      Client.request(client, \"eth_gasPrice\", [])\n    ]\n    {:ok, [block_number, gas_price]} = Client.send(client, requests)\n\nSee `Exth.Transport` for transport details and `Exth.Rpc.Request`\nfor request formatting.","ref":"Exth.Rpc.Client.html#module-examples"},{"type":"function","title":"Exth.Rpc.Client.new/2","doc":"","ref":"Exth.Rpc.Client.html#new/2"},{"type":"function","title":"Exth.Rpc.Client.request/2","doc":"","ref":"Exth.Rpc.Client.html#request/2"},{"type":"function","title":"Exth.Rpc.Client.request/3","doc":"","ref":"Exth.Rpc.Client.html#request/3"},{"type":"function","title":"Exth.Rpc.Client.send/2","doc":"","ref":"Exth.Rpc.Client.html#send/2"},{"type":"type","title":"Exth.Rpc.Client.send_argument_type/0","doc":"","ref":"Exth.Rpc.Client.html#t:send_argument_type/0"},{"type":"type","title":"Exth.Rpc.Client.send_response_type/0","doc":"","ref":"Exth.Rpc.Client.html#t:send_response_type/0"},{"type":"type","title":"Exth.Rpc.Client.t/0","doc":"","ref":"Exth.Rpc.Client.html#t:t/0"},{"type":"module","title":"Exth.Rpc.Request","doc":"Represents a JSON-RPC request with validation.\n\nA request consists of:\n  * `method` - The RPC method name (string or atom)\n  * `params` - List of parameters for the method\n  * `id` - Optional positive integer for request identification\n  * `jsonrpc` - JSON-RPC version (defaults to \"2.0\")","ref":"Exth.Rpc.Request.html"},{"type":"module","title":"Example - Exth.Rpc.Request","doc":"iex> Request.new(\"eth_getBalance\", [\"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\", \"latest\"])\n    %Request{\n      method: \"eth_getBalance\",\n      params: [\"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\", \"latest\"],\n      id: nil,\n      jsonrpc: \"2.0\"\n    }","ref":"Exth.Rpc.Request.html#module-example"},{"type":"function","title":"Exth.Rpc.Request.new/3","doc":"Creates a new request struct with the given method, parameters, and ID.","ref":"Exth.Rpc.Request.html#new/3"},{"type":"function","title":"Parameters - Exth.Rpc.Request.new/3","doc":"* `method` - The RPC method name (string or atom)\n  * `params` - List of parameters for the method\n  * `id` - Optional positive integer for request identification","ref":"Exth.Rpc.Request.html#new/3-parameters"},{"type":"function","title":"Returns - Exth.Rpc.Request.new/3","doc":"* A new request struct with the given method, parameters, and ID","ref":"Exth.Rpc.Request.html#new/3-returns"},{"type":"function","title":"Examples - Exth.Rpc.Request.new/3","doc":"# Simple request with no ID\n    request = Request.new(\"eth_blockNumber\", [])\n    # => %Request{id: nil, method: \"eth_blockNumber\", params: []}\n\n    # Request with ID\n    request = Request.new(\"eth_getBalance\", [\"0x742d...\", \"latest\"], 1)\n    # => %Request{id: 1, method: \"eth_getBalance\", params: [\"0x742d...\", \"latest\"]}","ref":"Exth.Rpc.Request.html#new/3-examples"},{"type":"function","title":"Exth.Rpc.Request.serialize/1","doc":"Serializes a request to JSON.","ref":"Exth.Rpc.Request.html#serialize/1"},{"type":"function","title":"Parameters - Exth.Rpc.Request.serialize/1","doc":"* `request` - The request to serialize","ref":"Exth.Rpc.Request.html#serialize/1-parameters"},{"type":"function","title":"Returns - Exth.Rpc.Request.serialize/1","doc":"* `{:ok, json}` - Successful serialization with JSON string\n  * `{:error, reason}` - Serialization failed with error reason","ref":"Exth.Rpc.Request.html#serialize/1-returns"},{"type":"function","title":"Examples - Exth.Rpc.Request.serialize/1","doc":"# Single request\n    {:ok, json} = Request.serialize(%Request{\n      method: \"eth_blockNumber\",\n      params: [],\n      id: 1\n    })\n\n    # Batch request\n    {:ok, json} = Request.serialize([\n      %Request{method: \"eth_blockNumber\", params: [], id: 1},\n      %Request{method: \"eth_getBalance\", params: [\"0x123...\", \"latest\"], id: 2}\n    ])","ref":"Exth.Rpc.Request.html#serialize/1-examples"},{"type":"type","title":"Exth.Rpc.Request.t/0","doc":"","ref":"Exth.Rpc.Request.html#t:t/0"},{"type":"module","title":"Exth.Rpc.Response","doc":"Represents JSON-RPC 2.0 response structures.\n\nA response can be either a `Success` or an `Error`:\n\n* `Success` - Contains the result of a successful RPC call\n  * `id` - Request identifier (matches the request)\n  * `result` - The actual response data\n  * `jsonrpc` - JSON-RPC version (defaults to \"2.0\")\n\n* `Error` - Contains error information when the RPC call fails\n  * `id` - Request identifier (matches the request)\n  * `error.code` - Integer error code\n  * `error.message` - Error description\n  * `error.data` - Optional additional error details\n  * `jsonrpc` - JSON-RPC version (defaults to \"2.0\")","ref":"Exth.Rpc.Response.html"},{"type":"module","title":"Examples - Exth.Rpc.Response","doc":"# Creating a success response\n    Response.success(1, \"0x1234\")\n    #=> %Response.Success{id: 1, result: \"0x1234\", jsonrpc: \"2.0\"}\n\n    # Creating an error response\n    Response.error(1, -32600, \"Invalid Request\")\n    #=> %Response.Error{\n      id: 1,\n      error: %{code: -32600, message: \"Invalid Request\", data: nil},\n      jsonrpc: \"2.0\"\n    }","ref":"Exth.Rpc.Response.html#module-examples"},{"type":"function","title":"Exth.Rpc.Response.deserialize/1","doc":"Deserializes a JSON-RPC response.","ref":"Exth.Rpc.Response.html#deserialize/1"},{"type":"function","title":"Examples - Exth.Rpc.Response.deserialize/1","doc":"iex> Exth.Rpc.Response.deserialize(~s({\"jsonrpc\": \"2.0\", \"result\": \"0x1234\", \"id\": 1}))\n    {:ok, %Exth.Rpc.Response.Success{id: 1, result: \"0x1234\"}}\n\n    iex> Exth.Rpc.Response.deserialize(~s({\"jsonrpc\": \"2.0\", \"error\": {\"code\": -32_601, \"message\": \"Method not found\"}, \"id\": 1}))\n    {:ok, %Exth.Rpc.Response.Error{id: 1, error: %{code: -32_601, message: \"Method not found\"}}}","ref":"Exth.Rpc.Response.html#deserialize/1-examples"},{"type":"function","title":"Exth.Rpc.Response.error/4","doc":"","ref":"Exth.Rpc.Response.html#error/4"},{"type":"function","title":"Exth.Rpc.Response.success/2","doc":"","ref":"Exth.Rpc.Response.html#success/2"},{"type":"type","title":"Exth.Rpc.Response.t/0","doc":"","ref":"Exth.Rpc.Response.html#t:t/0"},{"type":"module","title":"Exth.Rpc.Response.Error","doc":"Represents an error JSON-RPC 2.0 response.","ref":"Exth.Rpc.Response.Error.html"},{"type":"module","title":"Fields - Exth.Rpc.Response.Error","doc":"* `id` - Request identifier that matches the request ID\n  * `error` - A map containing error details:\n    * `code` - Integer error code (e.g., -32600 for \"Invalid Request\")\n    * `message` - Human-readable error description\n    * `data` - Optional additional error information\n  * `jsonrpc` - JSON-RPC version (defaults to \"2.0\")","ref":"Exth.Rpc.Response.Error.html#module-fields"},{"type":"module","title":"Example - Exth.Rpc.Response.Error","doc":"%Error{\n      id: 1,\n      error: %{\n        code: -32600,\n        message: \"Invalid Request\",\n        data: nil\n      },\n      jsonrpc: \"2.0\"\n    }\n\nCommon error codes:\n  * -32700: Parse error\n  * -32600: Invalid Request\n  * -32601: Method not found\n  * -32602: Invalid params\n  * -32603: Internal error","ref":"Exth.Rpc.Response.Error.html#module-example"},{"type":"type","title":"Exth.Rpc.Response.Error.t/0","doc":"","ref":"Exth.Rpc.Response.Error.html#t:t/0"},{"type":"module","title":"Exth.Rpc.Response.Success","doc":"Represents a successful JSON-RPC 2.0 response.","ref":"Exth.Rpc.Response.Success.html"},{"type":"module","title":"Fields - Exth.Rpc.Response.Success","doc":"* `id` - Request identifier that matches the request ID\n  * `result` - The actual response data from the RPC method call\n  * `jsonrpc` - JSON-RPC version (defaults to \"2.0\")","ref":"Exth.Rpc.Response.Success.html#module-fields"},{"type":"module","title":"Example - Exth.Rpc.Response.Success","doc":"%Success{\n      id: 1,\n      result: \"0x4b7\",\n      jsonrpc: \"2.0\"\n    }","ref":"Exth.Rpc.Response.Success.html#module-example"},{"type":"type","title":"Exth.Rpc.Response.Success.t/0","doc":"","ref":"Exth.Rpc.Response.Success.html#t:t/0"},{"type":"module","title":"Exth.Transport.Http","doc":"HTTP transport implementation for JSON-RPC requests using Tesla.\n\nImplements the `Exth.Transport.Transportable` protocol for making HTTP/HTTPS \nrequests to JSON-RPC endpoints. Uses Tesla as the HTTP client with Mint as the \ndefault adapter.","ref":"Exth.Transport.Http.html"},{"type":"module","title":"Usage - Exth.Transport.Http","doc":"transport = Transportable.new(\n      %Exth.Transport.Http{},\n      rpc_url: \"https://mainnet.infura.io/v3/YOUR-PROJECT-ID\",\n    )\n\n    {:ok, response} = Transportable.call(transport, request)","ref":"Exth.Transport.Http.html#module-usage"},{"type":"function","title":"Exth.Transport.Http.call/2","doc":"Makes an HTTP request to the JSON-RPC endpoint.\n\nReturns:\n  * `{:ok, responses}` - Successful request with decoded response or responses\n  * `{:error, {:http_error, status}}` - HTTP error response\n  * `{:error, reason}` - Other errors (network, timeout, etc)","ref":"Exth.Transport.Http.html#call/2"},{"type":"function","title":"Exth.Transport.Http.new/1","doc":"Creates a new HTTP transport with the given options.","ref":"Exth.Transport.Http.html#new/1"},{"type":"function","title":"Options - Exth.Transport.Http.new/1","doc":"* `:rpc_url` - (required) The HTTP/HTTPS endpoint URL\n  * `:adapter` - Tesla adapter to use (defaults to `Tesla.Adapter.Mint`)\n  * `:headers` - Additional HTTP headers for requests\n  * `:timeout` - Request timeout in milliseconds (defaults to 30000)","ref":"Exth.Transport.Http.html#new/1-options"},{"type":"type","title":"Exth.Transport.Http.t/0","doc":"HTTP transport configuration","ref":"Exth.Transport.Http.html#t:t/0"},{"type":"protocol","title":"Exth.Transport.Transportable","doc":"Protocol defining the interface for JSON-RPC transport implementations.\n\nThis protocol enables different transport mechanisms (HTTP, WebSocket, IPC, etc.)\nto be used interchangeably for making JSON-RPC requests.","ref":"Exth.Transport.Transportable.html"},{"type":"protocol","title":"Implementing the Protocol - Exth.Transport.Transportable","doc":"To implement a new transport, you need to define both `new/2` and `call/2`:\n\n    defimpl Transportable, for: MyTransport do\n      def new(transport, opts) do\n        # Initialize your transport with the given options\n      end\n\n      def call(transport, request) do\n        # Make the RPC request and return the response\n      end\n    end","ref":"Exth.Transport.Transportable.html#module-implementing-the-protocol"},{"type":"protocol","title":"Example Usage - Exth.Transport.Transportable","doc":"# Create a new transport instance\n    transport = Transportable.new(\n      %MyTransport{},\n      rpc_url: \"https://example.com/rpc\",\n    )\n\n    # Make an RPC request\n    {:ok, response} = Transportable.call(transport, request)","ref":"Exth.Transport.Transportable.html#module-example-usage"},{"type":"function","title":"Exth.Transport.Transportable.call/2","doc":"Makes a request using the configured transport.\n\nSupports both single requests and batch requests (arrays of requests).","ref":"Exth.Transport.Transportable.html#call/2"},{"type":"function","title":"Parameters - Exth.Transport.Transportable.call/2","doc":"* `transport` - The configured transport struct\n  * `request` - Single request or list of requests (Request.t() | [Request.t()])","ref":"Exth.Transport.Transportable.html#call/2-parameters"},{"type":"function","title":"Returns - Exth.Transport.Transportable.call/2","doc":"* `{:ok, response}` - Successful request with decoded response (Response.t())\n  * `{:ok, responses}` - Successful batch request with decoded responses ([Response.t()])\n  * `{:error, reason}` - Request failed with error reason (Exception.t() or map())","ref":"Exth.Transport.Transportable.html#call/2-returns"},{"type":"function","title":"Examples - Exth.Transport.Transportable.call/2","doc":"# Single request\n    {:ok, response} = Transportable.call(transport, %Request{\n      jsonrpc: \"2.0\",\n      method: \"eth_blockNumber\",\n      params: [],\n      id: 1\n    })\n\n    # Batch request\n    {:ok, responses} = Transportable.call(transport, [\n      %Request{method: \"eth_blockNumber\", params: [], id: 1},\n      %Request{method: \"eth_getBalance\", params: [\"0x123...\", \"latest\"], id: 2}\n    ])","ref":"Exth.Transport.Transportable.html#call/2-examples"},{"type":"function","title":"Exth.Transport.Transportable.new/2","doc":"Creates a new transport instance with the given options.","ref":"Exth.Transport.Transportable.html#new/2"},{"type":"function","title":"Parameters - Exth.Transport.Transportable.new/2","doc":"* `transport` - The transport struct to initialize\n  * `opts` - Keyword list of options specific to the transport implementation","ref":"Exth.Transport.Transportable.html#new/2-parameters"},{"type":"function","title":"Returns - Exth.Transport.Transportable.new/2","doc":"* The initialized transport struct","ref":"Exth.Transport.Transportable.html#new/2-returns"},{"type":"function","title":"Example - Exth.Transport.Transportable.new/2","doc":"transport = Transportable.new(\n      %MyTransport{},\n      rpc_url: \"https://example.com/rpc\",\n    )","ref":"Exth.Transport.Transportable.html#new/2-example"},{"type":"type","title":"Exth.Transport.Transportable.t/0","doc":"All the types that implement this protocol.","ref":"Exth.Transport.Transportable.html#t:t/0"},{"type":"extras","title":"Overview","doc":"# Exth\n\nExth is an Elixir client for interacting with EVM-compatible blockchain nodes\nvia JSON-RPC. It provides a robust, type-safe interface for making Ethereum RPC\ncalls.","ref":"readme.html"},{"type":"extras","title":"Features - Overview","doc":"- 🔒 **Type Safety**: Comprehensive type specs and validation\n- 🔄 **Transport Agnostic**: Pluggable transport system (HTTP, WebSocket, IPC)\n- 🎯 **Smart Defaults**: Sensible defaults with full configurability\n- 🛡️ **Error Handling**: Detailed error reporting and recovery\n- 📦 **Batch Support**: Efficient batch request processing\n- 🔌 **Protocol Compliance**: Full JSON-RPC 2.0 specification support\n- ⚙️ **Dynamic Configuration**: Flexible configuration through both inline options and application config","ref":"readme.html#features"},{"type":"extras","title":"Installation - Overview","doc":"Add `exth` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:exth, \"~> 0.1.0\"},\n    # Optional dependencies:\n    # Mint for Tesla adapter\n    {:mint, \"~> 1.7\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"Exth offers two ways to interact with EVM nodes:\n\n1. **Provider** (High-Level): Define a provider module with convenient function\n   names and no need to pass client references.\n2. **RPC Client** (Low-Level): Direct client usage with more control, requiring\n   explicit client handling.\n\n<!-- tabs-open -->","ref":"readme.html#usage"},{"type":"extras","title":"Provider (Recommended) - Overview","doc":"```elixir\n# Basic usage with inline configuration\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :your_otp_app,\n    transport_type: :http,\n    rpc_url: \"https://YOUR-RPC-URL\"\nend\n\n# Dynamic configuration through application config\n# In your config/config.exs or similar:\nconfig :your_otp_app, MyProvider,\n  rpc_url: \"https://YOUR-RPC-URL\",\n  timeout: 30_000,\n  max_retries: 3\n\n# Then in your provider module:\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :your_otp_app,\n    transport_type: :http\nend\n\n# Configuration is merged with inline options taking precedence\ndefmodule MyProvider do\n  use Exth.Provider,\n    otp_app: :your_otp_app,\n    transport_type: :http,\n    rpc_url: \"https://OVERRIDE-RPC-URL\" # This will override the config value\nend\n\n# Use the provider\n{:ok, block_number} = MyProvider.block_number()\n\n{:ok, balance} = MyProvider.get_balance(\n  \"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\",\n  \"latest\"\n)\n\n{:ok, block} = MyProvider.get_block_by_number(\"0x1\", true)\n\n{:ok, tx_hash} = MyProvider.send_raw_transaction(\"0x...\")\n```\n\nThe Provider approach is recommended for most use cases as it provides:\n\n- ✨ Clean, intuitive function names\n- 🔒 Type-safe parameters\n- 📝 Better documentation and IDE support\n- 🎯 No need to manage client references\n- ⚙️ Flexible configuration through both inline options and application config\n\n#### Configuration Options\n\nProviders can be configured through both inline options and application config.\nInline options take precedence over application config. Here are the available options:\n\n```elixir\n# Required options\ntransport_type: :http | :custom  # Transport type to use\nrpc_url: \"https://...\"          # RPC endpoint URL\n\n# Required inline option\notp_app: :your_otp_app          # Application name for config lookup\n\n# Custom transport options\nmodule: MyCustomTransport       # Required when transport_type is :custom\n\n# Optional http options\ntimeout: 30_000                 # Request timeout in milliseconds\nheaders: [{\"header\", \"value\"}]  # Custom headers for HTTP transport\n```","ref":"readme.html#provider-recommended"},{"type":"extras","title":"RPC Client - Overview","doc":"```elixir\nalias Exth.Rpc\n\n# 1. Define a client\n{:ok, client} = Rpc.new_client(\n  transport_type: :http,\n  rpc_url: \"https://YOUR-RPC-URL\"\n)\n\n# 2.1. Make RPC calls with explicit client\nrequest1 = Rpc.request(client, \"eth_blockNumber\", [])\n{:ok, block_number} = Rpc.send(client, request1)\n\n# 2.2. Or make RPC calls without a client\nrequest2 = Rpc.request(\n  \"eth_getBalance\",\n  [\"0x742d35Cc6634C0532925a3b844Bc454e4438f44e\", \"latest\"]\n)\n{:ok, balance} = Rpc.send(client, request2)\n\n# 3. You can also send multiple requests in one call\nrequests = [request1, request2]\n{:ok, responses} = Rpc.send(client, requests)\n\n# 4. You can invert the order of the arguments and pipe\nRpc.request(\"eth_blockNumber\", [])\n|> Rpc.send(client)\n\n# OR\n[request1, request2]\n|> Rpc.send(client)\n```\n\nUse the RPC Client approach when you need:\n\n- 🔧 Direct control over RPC calls\n- 🔄 Dynamic method names\n- 🛠️ Custom parameter handling\n- 🎛️ Flexible client management (multiple clients, runtime configuration)\n\n<!-- tabs-close -->","ref":"readme.html#rpc-client"},{"type":"extras","title":"Transport Options - Overview","doc":"Exth uses a pluggable transport system that supports different communication\nprotocols. Each transport type can be configured with specific options:\n\n<!-- tabs-open -->","ref":"readme.html#transport-options"},{"type":"extras","title":"HTTP Transport - Overview","doc":"The default HTTP transport is built on Tesla, providing a robust HTTP client\nwith middleware support:\n\n```elixir\n# Provider configuration\ndefmodule MyProvider do\n  use Exth.Provider,\n    transport_type: :http,\n    rpc_url: \"https://eth-mainnet.example.com\",\n    # Optional HTTP-specific configuration\n    adapter: Tesla.Adapter.Mint, # Default HTTP adapter\n    headers: [{\"authorization\", \"Bearer token\"}],\n    timeout: 30_000, # Request timeout in ms\nend\n\n# Direct client configuration\n{:ok, client} = Exth.Rpc.new(\n  transport_type: :http,\n  rpc_url: \"https://eth-mainnet.example.com\",\n  adapter: Tesla.Adapter.Mint,\n  headers: [{\"authorization\", \"Bearer token\"}],\n  timeout: 30_000\n)\n```\n\n- ✨ **HTTP** (`:http`)\n\n  - Built on Tesla HTTP client\n  - Configurable adapters (Mint, Hackney, etc.)\n  - Configurable headers and timeouts","ref":"readme.html#http-transport"},{"type":"extras","title":"Custom Transport - Overview","doc":"Implement your own transport by creating a module and implementing the\n`Exth.Transport.Transportable` protocol:\n\n```elixir\ndefmodule MyCustomTransport do\n  # Transport struct should be whatever you need\n  defstruct [:config]\nend\n\ndefimpl Exth.Transport.Transportable, for: MyCustomTransport do\n  def new(transport, opts) do\n    # Initialize your transport configuration\n    %MyCustomTransport{config: opts}\n  end\n\n  def call(transport, request) do\n    # Handle the JSON-RPC request\n    # Return {:ok, response} or {:error, reason}\n  end\nend\n\n# Use your custom transport\ndefmodule MyProvider do\n  use Exth.Provider,\n    transport_type: :custom,\n    module: MyCustomTransport,\n    rpc_url: \"custom://endpoint\",\n    # Additional custom options\n    custom_option: \"value\"\nend\n\n# Direct client configuration\n{:ok, client} = Exth.Rpc.new_request(\n  transport_type: :custom,\n  rpc_url: \"https://eth-mainnet.example.com\",\n  module: MyCustomTransport,\n  custom_option: \"value\"\n)\n```\n\n- 🔧 **Custom** (`:custom`)\n  - Full control over transport implementation\n  - Custom state management\n\n<!-- tabs-close -->","ref":"readme.html#custom-transport"},{"type":"extras","title":"Examples - Overview","doc":"Check out our [examples](https://github.com/joaop21/exth/tree/main/examples)\ndirectory for practical usage examples","ref":"readme.html#examples"},{"type":"extras","title":"Requirements - Overview","doc":"- Elixir ~> 1.18\n- Erlang/OTP 26 or later","ref":"readme.html#requirements"},{"type":"extras","title":"Contributing - Overview","doc":"1. Fork it\n2. Create your feature branch (`git checkout -b feature/my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin feature/my-new-feature`)\n5. Create new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - Overview","doc":"This project is licensed under the MIT License. See [LICENSE](LICENSE) for details.","ref":"readme.html#license"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2025 João\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"0.3.0 - 2025-04-11 - Changelog","doc":"","ref":"changelog.html#0-3-0-2025-04-11"},{"type":"extras","title":"Added - Changelog","doc":"- Support for dynamic configuration through both inline options and application configuration\n- examples directory with practical usage examples\n\n[0.3.0]: https://github.com/joaop21/exth/releases/tag/v0.3.0","ref":"changelog.html#added"},{"type":"extras","title":"0.2.2 - 2025-04-07 - Changelog","doc":"","ref":"changelog.html#0-2-2-2025-04-07"},{"type":"extras","title":"Changed - Changelog","doc":"- Moved `Exth.Rpc.Encoding.encode_request/1` logic to `Exth.Rpc.Request.serialize/1`\n- Moved `Exth.Rpc.Encoding.decode_response/1` logic to `Exth.Rpc.Response.deserialize/1`\n- Removed `Exth.Rpc.Encoding` module\n- Transport module does not receive an `encoder/1` and `decoder/1` option\n  anymore. It's now direct to call `Exth.Rpc.Request.serialize/1` and\n  `Exth.Rpc.Response.deserialize/1` functions.\n- These changes aim to reduce the complexity of `Exth.Rpc` and `Exth.Transport`\n  modules and make it easier to understand its internals.\n\n[0.2.2]: https://github.com/joaop21/exth/releases/tag/v0.2.2","ref":"changelog.html#changed"},{"type":"extras","title":"0.2.1 - 2025-04-05 - Changelog","doc":"","ref":"changelog.html#0-2-1-2025-04-05"},{"type":"extras","title":"Changed - Changelog","doc":"- `Exth.Provider` was refactored to reduce its complexity\n- Fixed dialyzer warnings on `Exth.Provider` related to `has no local return`.\n  Some types were fixed and the dialyzer warning does not appear anymore. No\n  more warnings when defining providers.\n\n[0.2.1]: https://github.com/joaop21/exth/releases/tag/v0.2.1","ref":"changelog.html#changed"},{"type":"extras","title":"0.2.0 - 2025-04-04 - Changelog","doc":"","ref":"changelog.html#0-2-0-2025-04-04"},{"type":"extras","title":"Added - Changelog","doc":"- Documentation for `Exth.Rpc.Request` and `Exth.Rpc.Response` structs\n- Allow inverting arguments order on `Exth.Rpc.send/2` and `Exth.Rpc.Client.send/2`\n  - This change allows for more flexible usage patterns (pipes)\n- Creation of request IDs on `Exth.Rpc.Client.send/2` when no pre-assigned IDs\n  are provided\n- Creation of requests with no pre-assigned IDs with `Exth.Rpc.request/2` and\n  `Exth.Rpc.Client.request/2`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Replaced `Jason` with `JSON` for JSON encoding/decoding\n- Updated `README.md` for documentation purposes\n- Updated `Exth.Rpc.Client` documentation\n\n[0.2.0]: https://github.com/joaop21/exth/releases/tag/v0.2.0\n\n## [0.1.1] - 2025-03-27","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- `mint` dependency is now `:optional` instead of `only: :dev`;\n- Updated `README.md` for documentation purposes, with more examples and better formatting;\n\n[0.1.1]: https://github.com/joaop21/exth/releases/tag/v0.1.1\n\n## [0.1.0] - 2025-03-25 🚀","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of Exth\n- Core JSON-RPC functionality\n  - Request/Response handling\n  - Method generation\n  - Parameter encoding\n- Transport layer abstraction\n  - HTTP Transport implementation using Tesla\n  - Configurable middleware support\n- Provider interface\n  - Ethereum namespace support (`eth_*` methods)\n  - Net namespace support (`net_*` methods)\n  - Web3 namespace support (`web3_*` methods)\n- Client caching mechanism\n- Comprehensive documentation\n- MIT License\n\n[0.1.0]: https://github.com/joaop21/exth/releases/tag/v0.1.0","ref":"changelog.html#added"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}